// Generated by CoffeeScript 1.12.5
define(["framework", "jquery", "underscore"], function(TWC, $, _) {

  /**
   * TTML parser plug-in
   * Parses subtitle files in the TTML format.
   * Times may be absolute to the timeline or relative
   *   Absolute times are ISO 8601 format (hh:mm:ss[.mmm])
   *   Relative times are a fraction followed by a unit metric (d.ddu)
   *     Relative times are relative to the time given on the parent node
   * Styling information is ignored
   * Xml is the file contents to be processed
   *
   * @param {Object} data
   *
   * Example:
    <tt xmlns:tts="http://www.w3.org/2006/04/ttaf1#styling" xmlns="http://www.w3.org/2006/04/ttaf1">
      <body region="subtitleArea">
        <div>
          <p xml:id="subtitle1" begin="0.76s" end="3.45s">
            It seems a paradox, does it not,
          </p>
        </div>
      </body>
    </tt>
   */
  var getMetricIndex, getMultipler, getNodeRegion, parseChildren, parseNode, rLineBreak, rValidXMLChars, rWhitespace, toSeconds;
  rWhitespace = /^[\s]+|[\s]+$/gm;
  rLineBreak = /(?:\r\n|\r|\n)/gm;
  rValidXMLChars = /[^\x09\x0A\x0D\x20-\xFF]/g;
  parseChildren = function(node, timeOffset, region) {
    var currNode, currRegion, newOffset, retVal;
    currNode = node.firstChild;
    currRegion = getNodeRegion(node, region);
    retVal = [];
    newOffset = void 0;
    while (currNode) {
      if (currNode.nodeType === 1) {
        if (currNode.nodeName === 'p') {
          retVal.push(parseNode(currNode, timeOffset, currRegion));
        } else if (currNode.nodeName === 'div') {
          newOffset = toSeconds(currNode.getAttribute('begin'));
          if (newOffset < 0) {
            newOffset = timeOffset;
          }
          retVal.push.apply(retVal, parseChildren(currNode, newOffset, currRegion));
        }
      }
      currNode = currNode.nextSibling;
    }
    return retVal;
  };
  getNodeRegion = function(node, defaultTo) {
    var region;
    region = node.getAttribute('region');
    if (region !== null) {
      return region;
    } else {
      return defaultTo || '';
    }
  };
  parseNode = function(node, timeOffset, region) {
    var sub;
    sub = {};
    sub.text = (node.textContent || node.text).replace(rWhitespace, '').replace(rLineBreak, '<br />').replace(rValidXMLChars, '');
    sub.id = node.getAttribute('xml:id') || node.getAttribute('id');
    sub.start = toSeconds(node.getAttribute('begin'), timeOffset);
    sub.end = toSeconds(node.getAttribute('end'), timeOffset);
    sub.target = getNodeRegion(node, region);
    if (sub.end < 0) {
      sub.end = toSeconds(node.getAttribute('duration'), 0);
      if (sub.end >= 0) {
        sub.end += sub.start;
      } else {
        sub.end = Number.MAX_VALUE;
      }
    }
    return sub;
  };
  toSeconds = function(t_in, offset) {
    var e, i;
    i = void 0;
    if (!t_in) {
      return -1;
    }
    try {
      return TWC.SDK.Player.toSeconds(t_in);
    } catch (error) {
      e = error;
      i = getMetricIndex(t_in);
      return parseFloat(t_in.substring(0, i)) * getMultipler(t_in.substring(i)) + (offset || 0);
    }
  };
  getMetricIndex = function(t_in) {
    var i;
    i = t_in.length - 1;
    while (i >= 0 && t_in[i] <= '9' && t_in[i] >= '0') {
      i--;
    }
    return i;
  };
  getMultipler = function(metric) {
    return {
      'h': 3600,
      'm': 60,
      's': 1,
      'ms': 0.001
    }[metric] || -1;
  };
  TWC.SDK.Player.prototype.parseSubtitle = function(xml) {
    var node, returnData;
    returnData = [];
    node = void 0;
    if (!xml || !xml.documentElement) {
      return returnData;
    }
    node = xml.documentElement.firstChild;
    if (!node) {
      return returnData;
    }
    while (node.nodeName !== 'body') {
      node = node.nextSibling;
    }
    if (node) {
      returnData = parseChildren(node, 0);
    }
    return returnData;
  };
});
